#version 330

uniform mat4 shadowMVMatrix;
uniform mat4 shadowPMatrix;
uniform sampler2D textureMap;
uniform float numberOfDirections;

in vec2 offset;
flat in vec3 center_projectedPos;
flat in vec3 center_Pos;
flat in float radius;

out vec4 out_color;

void main(void)
{


	//compute relative position on sphere (x,y,z) in local space
	vec2 offset_abs = abs(offset);
	float h = 1 - abs(offset[0]) - abs(offset[1]);
	vec3 relative_sphere_pos;
	if (h >= 0) {
		relative_sphere_pos = vec3(offset[0],offset[1], -h);
	}
	else {
		relative_sphere_pos = vec3(sign(offset[0]) * (1 - abs(offset[1])) , sign(offset[1]) * (1 - abs(offset[0])) , -h);
	}

	//compute absolute position on sphere in view space
	vec4 temp = shadowMVMatrix * vec4(center_Pos + (relative_sphere_pos * radius),1);

	//compute normal in view space
	vec4 temp2 = shadowMVMatrix*vec4(center_Pos,1);
	vec3 normal = normalize(temp.xyz-temp2.xyz);

	//apply projection matrix
	vec4 sphere_pos_w  = shadowPMatrix * temp;
	vec3 sphere_pos = sphere_pos_w.xyz/sphere_pos_w.w;

	//look up depth from depth map at computed xy position
	sphere_pos = sphere_pos * 0.5 + 0.5;
	float ShadowmapDepth = texture(textureMap, sphere_pos.xy).r;

	//compare computed z width depth value from depth map to see it is illuminated
	if (sphere_pos.z  <= ShadowmapDepth)
		//then write result to texture (use cosine between normal and light direction as weight
		out_color =  vec4(max(0, dot(normal,vec3(0,0,1))),0,0,0) / numberOfDirections;
	else
		out_color =  vec4(0,0,0,0);




}