// Fragment Shader  file "phongShader.frag"

#version 330

uniform mat4 modelViewMatrix;
uniform mat4 projectionMatrix;
uniform sampler2D textureMap;
uniform sampler2D shadowMap;
uniform vec2 contourData;
uniform bool shadowActive;
uniform bool occlusionActive;
uniform float ambientIntensity;
uniform float directIntensity;
uniform float glossiness;


uniform mat4 shadowPMatrix;
uniform float farPlane;

in vec3 lightDirection;
in vec3 cameraDirection;
in vec2 offset;
in vec4 shadowPos;
flat in float depth;
flat in float radius;
flat in vec3 color;
flat in vec4 quad;

in vec4 viewPos;

out vec4 out_color;

void main(void)
{
	
	float contour_width = contourData[0];
	float contour_variance = contourData[1];

	
	float distanceToCenter = dot(offset, offset);
	if (distanceToCenter>  1.0 + sqrt((-depth/(farPlane-0.1f))) * contour_width/radius) {   //eigentlich auch noch perspektivische projektion miteinbeziehen
		discard;
	} 
	

	if (distanceToCenter > 1) {

		out_color = vec4(0,0,0,1);
		vec4 depthvector = projectionMatrix * vec4(0,0,depth - (distanceToCenter-1)*contour_variance,1);
		gl_FragDepth = depthvector.z/depthvector.w;

	} else {

		//float temp = sqrt(1.0 - pow(distanceToCenter,2));
		float temp = sqrt(max(0,1.0 - pow(distanceToCenter,1)));
		//compute depth IMPORVE   //still not 100% correct
		//vec4 dtemp =  vec4(viewPos.xy,depth + (temp * radius),1);  //evtl x y auch erst jetz projizieren
		vec4 dtemp =  vec4(0,0,depth + (temp * radius),1);

		

		if (dtemp.z > -0.1) {
			vec4 dtemp =  vec4(0,0,-0.1,1);
			vec4 depthvector = projectionMatrix * dtemp;
			gl_FragDepth = depthvector.z/depthvector.w;
			out_color  = vec4(4* vec3(0.07,0.07,0.07) * color,1.0);
			return;
		}

		vec4 depthvector = projectionMatrix * dtemp;
		gl_FragDepth = depthvector.z/depthvector.w;

		//compute normal
		vec3 normal = vec3(offset, temp);
		vec3 normal_lookup = normalize(transpose(mat3(modelViewMatrix))*normal);

		vec3 lightDir = lightDirection;
		vec2 reflectionCoefficient = vec2(0.2,0.2);  //Hard coded reflection parameters (diffuse and specular)
		vec3 lightIntensity = directIntensity * vec3(1,1,1); // light color = white
		vec3 ambient;
		vec3 diffuse = vec3(0,0,0);
		vec3 specular = vec3(0,0,0);

		//Get precomputed ambient term from texture
		float d = abs(normal_lookup.x) + abs(normal_lookup.y) + abs(normal_lookup.z);
		vec2 ambient_texCoords;
		if (normal_lookup.z <= 0) {
			ambient_texCoords = vec2(normal_lookup.x/d,normal_lookup.y/d);
		} 
		else {
			ambient_texCoords = vec2(sign(normal_lookup.x)*(1-abs(normal_lookup.y)/d),sign(normal_lookup.y)*(1-abs(normal_lookup.x)/d));
		}
		ambient_texCoords = ambient_texCoords * 0.5 + 0.5;
		ambient_texCoords.x = mix(quad[0]+ 0.5 / 2048, quad[2] - 0.5 / 2048, ambient_texCoords.x); //ATTENTION TEXTURE SIZE HARD CODED!!!
		ambient_texCoords.y = mix(quad[1]+ 0.5 / 2048, quad[3] - 0.5 / 2048, ambient_texCoords.y);
		float ambient_occlusion = texture(textureMap, ambient_texCoords).r;

	
		//PHONG SHADING =============================================================

		//ambient part
		if (occlusionActive==true)
			ambient =  ambientIntensity * ambient_occlusion* color;
		else
			ambient =  ambientIntensity * vec3(0.07,0.07,0.07) * color;


		vec3 shadowPosition = (shadowPos.xyz/shadowPos.w)*0.5+0.5;
		float shadow = 1;
		float diff;
		if (shadowActive==true) {
		
		    diff = texture(shadowMap, shadowPosition.xy+vec2(0.5/2048,0.5/2048)).r - (shadowPosition.z);
			float shadow1  = diff > 0 ? 1 : 0;
			if (diff<0.005 && diff>0)
			shadow1 = diff/0.005;

			diff = texture(shadowMap, shadowPosition.xy+vec2(-0.5/2048,-0.5/2048)).r - (shadowPosition.z);
			float shadow2  = diff > 0 ? 1 : 0;
			if (diff<0.005 && diff>0)
			shadow2 = diff/0.005;

			diff = texture(shadowMap, shadowPosition.xy+vec2(-0.5/2048,0.5/2048)).r - (shadowPosition.z);
			float shadow3  = diff > 0 ? 1 : 0;
			if (diff<0.005 && diff>0)
			shadow3 = diff/0.005;

			diff = texture(shadowMap, shadowPosition.xy+vec2(0.5/2048,-0.5/2048)).r - (shadowPosition.z);
			float shadow4  = diff > 0 ? 1 : 0;
			if (diff<0.005 && diff>0)
			shadow4 = diff/0.005;
			
			shadow = (shadow1+shadow2+shadow3+shadow4)/4;
		}

		normal = normalize(normal);
		lightDir = normalize(lightDir);
	
		//diffuse part
		float N_L = dot(normal,lightDir);
		if (N_L > 0) {
		diffuse = shadow* reflectionCoefficient.x * ((color*lightIntensity) * N_L);

		//specular part
			float N_H = dot(normal,normalize(normalize(cameraDirection)+lightDir));
			if (N_H > 0) {
				float highlight = pow(N_H,glossiness);
				specular = shadow * reflectionCoefficient.y * (lightIntensity * highlight);
			}
		}
		out_color = vec4(ambient+diffuse+specular,1.0);
		
	}



	//============================================================================
}